//  
//  (c) June 14, 2008 Vasily Volkov
//  

#include "FFT.h"

__global__ void FFT64_device( float2 *work )
{	
    work += (blockIdx.y * gridDim.x + blockIdx.x) * 512;
    
    int tid = threadIdx.x;
    int hi = tid>>3;
    int lo = tid&7;

    //
    //  no sync in transpose is needed on G80 here
    //  since the permutations are within-warp
    //
    
    float2 a[8];
    __shared__ float smem[8*8*9];
    
    load( a, work + tid, 64 );
    
    FFT8( a );
    
    twiddle( a, lo, 64 );
    tranpose( a, &smem[hi*8*9+lo*9], 1, &smem[hi*8*9+lo], 9, 0 );
    
    FFT8( a );
    
    store( a, work + tid, 64 );
}	
    
extern "C" void FFT64( float2 *work, int batch )
{	
    FFT64_device<<< grid2D(batch/8), 64 >>>( work );
}	
